/****************************************************************/
/*								*/
/*								*/
/* (C) Copyright Motorola Inc, 1994.				*/
/*  All right reserved 						*/
/*                                                              */
/* Author: Uri Trichter                                         */
/*								*/
/* Description:							*/
/*	Simple Driver Set - Application Test routine 		*/
/*								*/
/*	This example is setting SCC_1 in LOOPBACK mode in	*/
/*	promisciuse transparent. The app is using the brg for	*/
/*	the rxclk and txclk and internal SYNC is selected.	*/
/*	There is only one buffer that is beeing transmitted	*/
/*	and the receive routine checks that it got the right 	*/
/* 	data. After that the receive procedure just release the	*/
/*	buffers it gets ( IDLE data 0xffff)			*/
/*								*/
/****************************************************************/

#include "simple.h"
#include "trans.h"
#include "setup.h"

#include <stdio.h>
#include <stdlib.h>

#define IMP_START_ADDR	0x700000L

#define MSG_LEN		0x20
#define SYNC_CHAR	0x20	/* space */

#define FALSE		0
#define TRUE		1

/* use a system call of the debugger to print out a character */
/* Microtec printf uses this function */
void _OUTCHR( char ch)
{
	asm("	MOVE.L   $4(A7),D0");
	asm("	MOVE.B   D0,-(A7)");
	asm("	TRAP	 #15");
	asm("	DC.W	 $20");
	asm("	RTS");
}

trap15()
{
	asm("	trap    #15");
	asm("	dc.w    $63");
}

void got_a_buffer(int scc_number, void *buff, int length)
{
	int i;
	static got_a_buff = FALSE;

	if(!got_a_buff) {
		printf("Got the Sync\n");
		for( i=1; i<length-1; i++ ){
			if(((unsigned char *)buff)[i] != 'A'+i-1)
				printf("Wrong Input\n"); 
	
		}
		got_a_buff = TRUE;
	}
	free(buff);
}

void rx_errors(int scc_number, IMP_BD *rbd)
{
	/* just record statistics about receive errors that accure */
	static int cd = 0;

	if(rbd->status & R_CD) 
		 cd++;
	if(rbd->status & R_OV)
		/* abort the running */
		printf("Overrun Error !! Aborting\n");
		trap15();

}

void tx_errors(int scc_number, IMP_BD *tbd)
{
	printf("Transmit Error\n");
}

void busy_errors(int scc_number)
{
	IMP_BD	*bd;

	/* first check if it is a problem becuase the receiver
	   Can not malloc memory to replenish Rx buffers.
	*/

	bd = scc_table[scc_number].rbd;
	if( bd->length == 0 ){
		printf("System Can not allocate memory!! Check Heap Size\n");
		/* try to allocate memory again */
		alloc_mem(scc_number,bd);
	}else 
		printf("You Have A Busy Condition!! Check Performance \n");
		
}


/* write user interrupt service routine */
/* This is Microtech way to end a routine with RTE command */ 

void interrupt scc_isr_0()
{
	scc_interrupt( SCC_0 );
}

void interrupt scc_isr_1()
{
	scc_interrupt( SCC_1 );
}

void interrupt scc_isr_2()
{
	scc_interrupt( SCC_2 );
}


main()
{

	TRANS_SETUP set;
	unsigned char *buf; 
	int j,i;

	/* initiate IMP */
	imp_init(IMP_START_ADDR);
	
	printf("Hello World\n");
	
	/* init the scc */

	scc_init( SCC_1, 	
		MSG_LEN,
		malloc,
		free,
		got_a_buffer,
		tx_errors,
		rx_errors,
		busy_errors); 
*(short *)(0x7008ee)= 0x0100;

	/* init setup structure */

	set.baud = 9600;
	set.reverse_data = FALSE;
	set.promiscuous = TRUE;
	set.externl_sync = FALSE;
	set.sync = (SYNC_CHAR<<8)|SYNC_CHAR;
	set.diag_bits = NORMAL_SW;
	set.ex_rx_clk_source =FALSE;
	set.ex_tx_clk_source = FALSE;

  	/* init cannel to be UART */ 
	trans_init(SCC_1, &set);

	if(( buf = malloc(MSG_LEN)) == 0)
		printf("Alloc Problems\n");
	else {
		buf[1]= buf[0]= SYNC_CHAR;
		for(i=2; i<MSG_LEN; i++) 
			buf[i]='A'+i-2;

		/* send the buf */
		while( trans_tx(SCC_1, buf, MSG_LEN) == BUSY )
				;
	}

	for(;;)
	;
}
