#ifndef __IMP
#define __IMP
/*
 * imp.h
 * definitions of IMP memory structures
 */

/*
 * include registers and
 * parameter ram definitions files
 */
#include "registers.h"
#include "pram.h"


/* define system clock rate */
#define SYSTEM_CLOCK    20000000L
#define MAX_DIV		0x7ff	/* Maximum Devidor */

/* Buffer Descriptors */
typedef struct imp_bd {
	unsigned short	status;
	unsigned short	length;
	unsigned char	*buf;
} IMP_BD;

/*
 * transmit BD's status (common to all modes)
 */
#define	T_R	0x8000		/* ready bit */
#define	T_X	0x4000		/* extern buffer */
#define	T_W	0x2000		/* wrap bit */
#define	T_I	0x1000		/* interrupt on completion */

/*
 * receive BD's status (common to all modes)
 */
#define	R_E	0x8000		/* buffer empty */
#define	R_X	0x4000		/* external buffer */
#define	R_W	0x2000		/* wrap bit */
#define	R_I	0x1000		/* interrupt on reception */



/*
 * internal ram
 */
typedef struct imp {
/* BASE + 0x000: user data memory */
	unsigned char	udata[0x240];	/* 0x240 bytes user data */
	unsigned char	uempty[0x1c0];	/* empty till 0x400 */

/* BASE + 0x400: PARAMETER RAM */
	union {
		struct scc_pram {
			IMP_BD	rbd[8];
			IMP_BD	tbd[8];
			union {
				struct hdlc_pram	h;
				struct uart_pram	u;
				struct bisync_pram	b;
				struct _16550_pram	d;
				unsigned char		sc[0x40];
			} pscc;		/* scc parameter area (protocol dependent) */
		} scc;
		unsigned char	pr[0x100];
	} pram[3];
/* reserved */
	unsigned char	resrvd1[0x100];

/* BASE + 0x800: INTERNAL REGISTERS */
/* DMA */
	unsigned short	dma_res1;	/* reserved */
	unsigned short	dma_mode;	/* dma mode reg */
	unsigned long	dma_s;		/* dma source */
	unsigned long	dma_d;		/* dma destination */
	unsigned short	dma_cnt;	/* dma byte count */
	unsigned char	dma_status;	/* dma status */
	unsigned char	dma_res2;	/* reserved */
	unsigned char	dma_fc;		/* dma function code */
	unsigned char	dma_res3;	/* reserved */
/* Interrupt Controller */
	unsigned short	intr_mode;	/* interrupt mode register */
	unsigned short	intr_pending;	/* interrupt pending register */
	unsigned short	intr_mask;	/* interrupt mask register */
	unsigned short	intr_inservice;	/* interrupt in service register */
	unsigned short	intr_res;	/* reserved */
	unsigned short	intr_res1;	/* reserved */
/* Parallel I/O */
	unsigned short	pa_control;	/* port A control */
	unsigned short	pa_direction;	/* port A data direction */
	unsigned short	pa_data;	/* port A data register */
	unsigned short	pb_control;	/* port B control */
	unsigned short	pb_direction;	/* port B data direction */
	unsigned short	pb_data;	/* port B data */
	unsigned short	p_res;		/* reserved */
/* Chip Select */
	unsigned short	cs_res;
	unsigned short	cs_res1;
	unsigned short	cs0_base;	/* base address 0 */
	unsigned short	cs0_option;	/* option register 0 */
	unsigned short	cs1_base;
	unsigned short	cs1_option;
	unsigned short	cs2_base;
	unsigned short	cs2_option;
	unsigned short	cs3_base;
	unsigned short	cs3_option;
/* Timer */
	unsigned short	t0_mode;	/* timer 0 mode register */
	unsigned short	t0_ref;		/* timer 0 reference register */
	unsigned short	t0_cap;		/* timer 0 capture register */
	unsigned short	t0_counter;	/* timer 0 counter */
	unsigned char	tim_res0;	/* reserved */
	unsigned char	t0_status;	/* timer 0 status */

	unsigned short	wd_ref;		/* watch dog timer reference */
	unsigned short	wd_counter;	/* watch dog counter */
	unsigned short	tim_res2;	/* reserved */

	unsigned short	t1_mode;	/* timer 1 mode register */
	unsigned short	t1_ref;		/* timer 1 reference register */
	unsigned short	t1_cap;		/* timer 1 capture register */
	unsigned short	t1_counter;	/* timer 1 counter */
	unsigned char	tim_res3;	/* reserved */
	unsigned char	t1_status;	/* timer 1 status */
	unsigned short	tim_res4[3];	/* reserved */
/* command register */
	unsigned char	cr;		/* command register */
	unsigned char	cr_res;		/* reserved */
/* reserved */
	unsigned char	resrvd2[0xe];
/* PCMCIA registers	 Base + 870 */
	unsigned char	cor;		/* Configuration Option Register	*/
	unsigned char	ccsr;		/* Card Configuration and Status Register*/
	unsigned char   prr;		/* Pin Replacement Register	*/
	unsigned char   scr;		/* Socket and Copy Register	*/
	unsigned char   ioei;		/* I/O Event indication register */

/* reserved */
	unsigned char	resrvd3[0xb];
/* SCC registers */
	struct scc_regs {
		unsigned short	resvd;	/* reserved */
		unsigned short	conf;	/* SCC configuration register */
		unsigned short	mode;	/* SCC mode register */
		unsigned short	sync;	/* SCC sync register */
		unsigned short	event;	/* SCC event register */
		/* unsigned char	res1; */	/* reserved */
		unsigned short	mask;	/* SCC mask register */
		/* unsigned char	res2; */	/* reserved */
		unsigned short	status;	/* SCC status register */
		/* unsigned char	res3;*/	/* reserved */
		unsigned short	res4;	/* reserved */
	} scc_regs[3];
/* SP (SCP + SMI) */
	unsigned short	sp_scm;		/* scp, smi mode + clock control */
/* Serial Interface */
	unsigned short	si_mask;	/* mask register */
	unsigned short	si_mode;	/* mode register */

/* reserved */
	unsigned char	resrvd4[0xa];

/* PCMCIA configuration registers cont.  addr 8c0*/
	unsigned long 	pcmcia_pcmr;	/* PCMCIA Mode Register */
	unsigned char	pcmcia_pcrwer;	/* PCMCIA Configuration Registers Write Event Register */ 
	unsigned char 	pcmcia_pcawer; /*PCMCIA Access Wake-Up Event Register */	
	unsigned char	pcmcia_pcrwmr;	/* PCMCIA Configuration Registers Write Mask Register */
	unsigned char 	pcmcia_pcawmr; /*PCMCIA Access Wake-Up Event Mask Register */	
	unsigned char   pcmcia_pcher; /*PCMCIA Host (PC) Event Register - PCHER */
	unsigned char   pcmcia_res1[3];
	unsigned short	cisbar;		/* CIS base address register */
	unsigned short	pcmcia_res2;
	unsigned short	cmbar1; 	/*Common Memory Space Base address reg1 */
	unsigned short	pcmcia_res3;
	unsigned short	cmbar2;		/* Common Memory Space Base address reg2 */
	unsigned short 	pcmcia_res4;

	unsigned char 	pcmcia_res6[0x08];
/* PORT C and PORT D Register */
	unsigned char	pucr;	/* Pull-Up Control Register */
	unsigned char 	pcmcia_res7;
	unsigned char	pcddr;	/* Port C data direction register */
	unsigned char 	pcmcia_res8;
	unsigned char	pcdat;	/* Port C data register */
	unsigned char 	pcmcia_res9;
	unsigned char	pddat;	/* Port D data register */
	unsigned char 	pcmcia_res10;
	
/* DSP Interface  Base + 8e8*/
	unsigned short	hbar;		/* DSP host port base address register */
	unsigned char	docr;		/* DSP host port configuration option register */
	unsigned char	dsp_res1[3];
	unsigned short	icr;		/* Internal Connection Register */

/* 16550 Emulation */
	unsigned short 	emr;		/* 16550 Mode register */
	unsigned char 	lcr;		/* 16550 line control register */
	unsigned char 	msr;		/* 16550 modem status register */
	unsigned char 	dlm;		/* 16550 divisor latch (MSB) */
	unsigned char 	dll;		/* 16550 divisor latch (LSB) */
	unsigned char 	lsr;		/* 16550 line status register */

/* reserved */
	unsigned char	resrvd5[0x708];
} IMP;

#endif
