#ifndef __PRAM
#define __PRAM
/*
 * pram.h
 * Definitions of the parameter area RAM.
 * Note that different structures are overlaid
 * at the same offsets for the different modes
 * of operation.
 */

/*****************************************************************
	HDLC parameter RAM
*****************************************************************/

struct hdlc_pram {
	unsigned char	rfc;		/* Rx FC register */
	unsigned char	tfc;		/* Tx FC register */
	unsigned short	rblen;		/* Rx buffer length */
	unsigned short	rstate;		/* Rx internal state */
	unsigned short	rbuf;		/* Rx buffer number */
	unsigned long	rptr;		/* Rx internal data pointer */
	unsigned short	rcount;		/* Rx byte count */
	unsigned short	rtemp;		/* Rx temp */

	unsigned short	tstate;		/* Tx internal state */
	unsigned short	tbuf;		/* Tx buffer number */
	unsigned long	tptr;		/* Tx internal data pointer */
	unsigned short	tcount;		/* Tx byte count */
	unsigned short	ttemp;		/* Tx temp */

	unsigned short	rcrc1;		/* CRC low word (Rx) */
	unsigned short	rcrc2;		/* CRC high word (Rx) */
	unsigned short	r_cmaskl;	/* constant */
	unsigned short	r_cmaskh;	/* constant */
	unsigned short	tcrc_l;		/* CRC low word (Tx) */
	unsigned short	tcrc_h;		/* CRC high word (Tx) */

	unsigned short	discard_cnt;	/* discarded frames counter */
	unsigned short	crc_cnt;	/* CRC error counter */
	unsigned short	abort_cnt;	/* rx-ed abort counter */
	unsigned short	nmf_cnt;	/* not my frame counter */
	unsigned short	retx_cnt;	/* retransmission counter */

	unsigned short	r_max;		/* maximum frame length */
	unsigned short	r_maxcnt;	/* maximum frame counter */
	unsigned short	r_mask;		/* address mask */
	unsigned short	r_addr1;	/* address 1 */
	unsigned short	r_addr2;	/* address 2 */
	unsigned short	r_addr3;	/* address 3 */
	unsigned short	r_addr4;	/* address 4 */
};



/*****************************************************************
	UART parameter RAM
*****************************************************************/

/*
 * bits in uart control characters table
 */
#define	CC_INVALID	0x8000		/* control character is valid */
#define	CC_REJ		0x4000		/* don't store char in buffer */
#define	CC_CHAR		0x00ff		/* control character */
#define REA		0x2000
#define CC_I		0x1000
#define CC_CT		0x0800
#define CC_A		0x0100

/* UART */
struct uart_pram {
	unsigned char	rfc;		/* Rx FC register */
	unsigned char	tfc;		/* Tx FC register */
	unsigned short	rblen;		/* Rx buffer length */
	unsigned short	rstate;		/* Rx internal state */
	unsigned short	rbuf;		/* Rx buffer number */
	unsigned long	rptr;		/* Rx internal data pointer */
	unsigned short	rcount;		/* Rx byte count */
	unsigned short	rtemp;		/* Rx temp */

	unsigned short	tstate;		/* Tx internal state */
	unsigned short	tbuf;		/* Tx buffer number */
	unsigned long	tptr;		/* Tx internal data pointer */
	unsigned short	tcount;		/* Tx byte count */
	unsigned short	ttemp;		/* Tx temp */

	unsigned short	maxi;		/* maximum idle characters */
	unsigned short	rxidle;		/* rx idle counter (internal) */
	unsigned short	brk;		/* break count register */

	unsigned short	parity_cnt;	/* Rx parity error counter */
	unsigned short	framing_cnt;	/* Rx framing error counter */
	unsigned short	noise_cnt;	/* Rx noise counter */
	unsigned short	break_cnt;	/* Rx break character counter */

	unsigned short	r_addr1;	/* address 1 */
	unsigned short	r_addr2;	/* address 2 */
	unsigned short	ccr;		/* Rx control char register */
	unsigned short	cc[8];		/* Rx control characters */
};



/*****************************************************************
	BISYNC parameter RAM
*****************************************************************/

struct bisync_pram {
	unsigned char	rfc;		/* Rx FC register */
	unsigned char	tfc;		/* Tx FC register */
	unsigned short	rblen;		/* Rx buffer length */
	unsigned short	rstate;		/* Rx internal state */
	unsigned short	rbuf;		/* Rx buffer number */
	unsigned long	rptr;		/* Rx internal data pointer */
	unsigned short	rcount;		/* Rx byte count */
	unsigned short	rtemp;		/* Rx temp */

	unsigned short	tstate;		/* Tx internal state */
	unsigned short	tbuf;		/* Tx buffer number */
	unsigned long	tptr;		/* Tx internal data pointer */
	unsigned short	tcount;		/* Tx byte count */
	unsigned short	ttemp;		/* Tx temp */

	unsigned short	rcrc;		/* temp Rx CRC */
	unsigned short	crc_c;		/* CRC constant */
	unsigned short	prcrc;		/* preset rx CRC value */
	unsigned short	tcrc;		/* temp Tx CRC */
	unsigned short	ptcrc;		/* preset tx CRC value */

	unsigned short	res1;
	unsigned short	res2;
	unsigned short	parity_cnt;	/* Rx parity error counter */

	unsigned short	sync;		/* SYNC character */
	unsigned short	dle;		/* DLE character */
	unsigned short	cc[8];		/* control characters */
};

/*****************************************************************
	16550 parameter RAM
*****************************************************************/

struct _16550_pram {
	unsigned char	rfc;		/* Rx FC register */
	unsigned char	tfc;		/* Tx FC register */
	unsigned short	rblen;		/* Rx buffer length */
	unsigned short	rstate;		/* Rx internal state */
	unsigned short	rbuf;		/* Rx buffer number */
	unsigned long	rptr;		/* Rx internal data pointer */
	unsigned short	rcount;		/* Rx byte count */
	unsigned short	rtemp;		/* Rx temp */

	unsigned short	tstate;		/* Tx internal state */
	unsigned short	tbuf;		/* Tx buffer number */
	unsigned long	tptr;		/* Tx internal data pointer */
	unsigned short	tcount;		/* Tx byte count */
	unsigned short	ttemp;		/* Tx temp */

	unsigned short	maxi;		/* maximum idle characters */
	unsigned short	idlc;		/* idle counter (internal) */
	unsigned short	brkc;		/* break count register */

	unsigned short	rsrv[3];	/* reserved no use	*/
	unsigned short	brkec;		/* Receive brk condition cnt */ 
	unsigned short	rsrv1[2];	/* reserved no use	*/

	unsigned short	rccr;		/* Rx control char register */
	unsigned short	cc[8];		/* Rx control characters */
};

/*****************************************************************
	IOM2 parameter RAM
	(overlaid on tx bd[5] of SCC channel[2])
*****************************************************************/
struct iom2_pram {
	unsigned short	ci_data;	/* ci data */
	unsigned short	monitor_data;	/* monitor data */
	unsigned short	tstate;		/* transmitter state */
	unsigned short	rstate;		/* receiver state */
};

/*****************************************************************
	SCP/SMI parameter RAM
	(overlaid on tx bd[6,7] of SCC channel[2])
*****************************************************************/

#define	SCP_R	0x8000		/* Ready bit in BD */

struct scp_smi_pram {
	unsigned short	res[3];		/* reserved */
	unsigned short	smi0_r;		/* smi 0 Rx BD */
	unsigned short	smi0_t;		/* smi 0 Tx BD */
	unsigned short	smi1_r;		/* smi 1 Rx BD */
	unsigned short	smi1_t;		/* smi 1 Tx BD */
	unsigned short	internal[6];	/* for internal use only */
	unsigned short	scp_bd;		/* Tx/Rx BD */
	unsigned short	berr;		/* bus error channel number */
	unsigned short	rev_number;	/* chip revision number */
};
#endif
