#ifndef __REGISTERS
#define __REGISTERS
/*
 * registers.h
 * Definitions of the IMP registers
 */


#ifdef SPV_WRITE
/* define size of data	*/
#define BYTE_SIZE	0x00
#define WORD_SIZE	0x01
#define LONG_SIZE	0x02

void write_in_spv_mode( int address, int data, int size );
#endif

/*****************************************************************
	Base Address Register (address 0xf2)
*****************************************************************/

#ifdef SPV_WRITE
#define	WRITE_BAR(x)	write_in_spv_mode(0xf2,x,WORD_SIZE)
#else
#define	WRITE_BAR(x)	(*((short *)0xf2) = x) 
#endif


/*****************************************************************
	System Control Register (address 0xf4)
*****************************************************************/

#ifdef SPV_WRITE
#define	WRITE_SCR(x)	write_in_spv_mode(0xf4,x,LONG_SIZE)
#define WRITE_IOMCR(x)  write_in_spv_mode(0xfa,x,BYTE_SIZE)
#define WRITE_IPDR(x)  	write_in_spv_mode(0xfb,x,BYTE_SIZE)
#define WRITE_PITR(x)  	write_in_spv_mode(0xf0,x,WORD_SIZE)
#define WRITE_IPLCR(x) 	write_in_spv_mode(0xf8,x,WORD_SIZE)
#define WRITE_IWUCR(x) 	write_in_spv_mode(0xf7,x,BYTE_SIZE)
#else
#define	WRITE_SCR(x)	(*((long *)0xf4) = x)
#define	CLEAR_IPA	(*((long *)0xf4) |= 0x08000000L)
#define WRITE_IOMCR(x)  (*((char *)0xfa) = x)
#define WRITE_IPDR(x)  	(*((char *)0xfb) = x)
#define WRITE_PITR(x)  	(*((short *)0xf0) = x)
#define WRITE_IPLCR(x) 	(*((short *)0xf8) = x)
#define WRITE_IWUCR(x) 	(*((char *)0xf7) = x)
#endif

#define	CLEAR_IPA	(*((long *)0xf4) |= 0x08000000L)
#define READ_IOMCR 	(*((char *)0xfa)) 
#define READ_IWUCR 	(*((char *)0xf7)) 
#define CLEAR_BIT(x, bit)       x = bit

/* IOMCR Low Power bits */
#define NORMAL_MODE 	0x00
#define STAND_BY_MODE	0x01
#define DOZE_MODE	0x02
#define STOP_MODE	0x03	

/* IMP Wake-Up Control Register- IWUCR BITS */
#define PITEv		0x40
#define PB10Ev		0x20
#define PB9Ev		0x10
#define PITEn		0x04
#define PB10En		0x02
#define PB9En		0x01


/*****************************************************************
	Command Register
*****************************************************************/

/* bit fields within command register */
#define	SOFTWARE_RESET	0x80
#define	CMD_GCI		0x40
#define	CMD_OPCODE	0x30
#define	CMD_CHANNEL	0x06
#define	CMD_FLAG	0x01

/* command opcodes */
#define	STOP_TX		0x00
#define	RESTART_TX	0x10
#define	ENTER_HUNT_MODE	0x20
#define	RESET_RX_BCS	0x30	/* bisync only */
#define	GCI_ABORT	CMD_GCI | 0x04
#define	GCI_TIMEOUT	CMD_GCI | 0x14




/***************************************************************
	SCC Configuration register
*****************************************************************/

#define WOMS	0x8000
#define EXTC	0x4000	/* external clock source */
#define TCS	0x2000	/* transmit clock source */
#define RCS	0x1000	/* receive clock source */

/*****************************************************************
	SCC mode register
*****************************************************************/

/* SCC modes */
#define	HDLC_PORT		0
#define ASYNC_PORT		1	/* uart */
#define	_16550_PORT		2
#define	BISYNC_PORT		3
#define	PROM_TRANS		3

/* 
 * enable rx tx 
 */

#define ENABLE_TX	(1<<2)
#define ENABLE_RX	(1<<3)

/*
 * diagnostics bits
 */
#define	NORMAL		(0<<4)	/* normal operation with CTS and CD */
				/* automatically controlled by SCC */
#define	LOOPBACK	(1<<4)	/* connect tx to rx */
#define	SERIAL_ECHO	(2<<4)	/* connect rx to tx */
#define	NORMAL_SW	(3<<4)	/* normal operation with CTS and CD */
				/* under control of s/w */

/* common mode register bits */
#define	MODE		0x0003

/* hdlc mode register */
#define	CRC32		0x0800


/* uart mode register */

#define	EN_PARITY	(1<<12)
#define	RCV_ODD_PARITY	(0<<13)
#define	RCV_EVEN_PARITY	(1<<13)
/*
 * tx parity (SCC asynchronous)
 */
#define	TX_ODD_PARITY	(0<<15)
#define	TX_EVEN_PARITY	(1<<15)
#define	TX_LOW_PARITY	(2<<15)
#define	TX_HIGH_PARITY	(3<<15)

/*
 * UART options
 */
#define	UOPT		0x0c00
#define	OPT_NORMAL	(0<<10)
#define	OPT_MULTD	(1<<10)	/* multi-drop, no auto */
#define	OPT_DDCMP	(2<<10)	/* ddcmp */
#define	OPT_AUTO	(3<<10)	/* mutli-drop with automatic */
				/* address recognition */
#define	UART_FRZ	0x0200

/* Character Length */
#define SEVEN_BIT_CHAR 	(0<<8)
#define EIGHT_BIT_CHAR 	(1<<8)

/* number of stop bits */
#define ONE_STOP_BIT	(0<<6)
#define TWO_STOP_BITS	(1<<6)

/* Set the fractional stop bits */
#define WHOLE_STOP_BIT	(7<<12)
#define SEVEN_EIGTH_STOP_BIT (5<<12) 
#define THREE_QUATERS_STOP_BIT	(3<<12) 

/* bisync/transparent mode register */
#define TR_EXSYN	0x4000
#define TR_NTSYM	0x2000
#define TR_REVD		0x1000
#define	BI_CRC16	0x0800
#define	BI_TRANSP	0x0200
#define	BI_BCS		0x0100

/* 16550 status reg */
#define LOOP_STAT	0x0020
#define ID_STAT		0x0010
#define OUT2_STAT	0x0008
#define OUT1_STAT	0x0004
#define RTS_STAT	0x0002
#define DTR_STAT	0x0001

/*****************************************************************
	SCP/SMI mode register
*****************************************************************/

/*
 * defines for SMI modes
 */
#define	IOM_UNUSED	0	/* IOM: monitor not used */
#define	IOM_DCHAN	1	/* IOM: monitor controls D channel */
#define	IOM_DATA_CNTRL	2	/* IOM: monitor txs data/control bytes */
#define	IDL_AM_HUNT	4	/* IDL: hunt M and A channels on zero */
#define	IDL_M_HUNT	5	/* IDL: hunt M channel on zero */
#define	IDL_A_HUNT	6	/* IDL: hunt A channel on zero */
#define	IDL_NORMAL	7	/* IDL: normal (no hunt on zero mode) */

#define	SCP_START	0x8000


/*****************************************************************
	serial interface mode register
*****************************************************************/

/*
 * defines for D/B channel routing
 * (b2_rout, b1_rout, d_rout)
 */
#define	NOT_SUPPORTED	0
#define	ROUTE_SCC0	1
#define	ROUTE_SCC1	2
#define	ROUTE_SCC2	3

/*
 * serial interface mode
 */
#define	IDL_MODE	0
#define	IOM_MODE	1
#define	PCM_MODE	2
#define	NMSI_MODE	3


/*****************************************************************
	serial interface mask register
*****************************************************************/



/*****************************************************************
	interrupt registers
*****************************************************************/

#define	INTR_PB11	0x8000		/* parallel port B bit 11 */
#define	INTR_PB10	0x4000		/* parallel port B bit 10 */
#define	INTR_SCC0	0x2000		/* SCC port 0 */
#define	INTR_DMAERR	0x1000		/* dma error */
#define	INTR_DMA	0x0800		/* dma */
#define	INTR_SCC1	0x0400		/* SCC port 1 */
#define	INTR_TIMER0	0x0200		/* timer 0 */
#define	INTR_SCC2	0x0100		/* SCC port 2 */
#define	INTR_PB9	0x0080		/* parallel port B bit 9 */
#define	INTR_TIMER1	0x0040		/* timer 1 */
#define	INTR_SCP	0x0020		/* SCP port */
#define	INTR_TIMER2	0x0010		/* timer 2 */
#define	INTR_SMI0	0x0008		/* SMI port 0 */
#define	INTR_SMI1	0x0004		/* SMI port 1 */
#define	INTR_PB8	0x0002		/* parallel port B bit 8 */
#define	INTR_ERR	0x0001		/* error */




/*****************************************************************
	dma mode register
*****************************************************************/

#define	DMA_EXT		0x4000
#define	REQG		0x0c00
#define	DMA_RESET	0x0002
#define	DMA_START	0x0001

/*
 * request generation
 */
#define	EXTERNAL_REQG	0x0800
#define	INTREQ_LIM	0x0000
#define	INTREQ_MAX	0x0400
#define	EXTREQ_BURST	0x0800
#define	EXTREQ_CYCL	0x0c00

/*****************************************************************
	chip select option register
*****************************************************************/
#define	DTACK		0xe000
#define	ADR_MASK	0x1ffc
#define	RW_MASK		0x0002
#define	FC_MASK		0x0001


/*****************************************************************
	16550 register's
*****************************************************************/
/* emr - emulation mode register */
#define RSDE		0x0001
#define TSDE		0x0002
#define RX_DIV		0x003c
#define TX_DIV		0x03c0
#define _16550_ENR	0x0400
#define _16550_ENT	0x0800
#define FRZ		0x1000
#define NWAIT		0x2000

/* lcr register */
#define CHARACTER_LEN	0x03
#define STOP_BIT_LENGTH	0x04
#define PARITY_ENABLE	0x08
#define LCR_EVEN_PARITY	0x10
#define STICK_BIT	0x20
#define SET_BRK 	0x40
#define DLAB	 	0x80

/*****************************************************************
	PCMCIA register's
*****************************************************************/
/* 3 even address PCMCIA mode registers */
#define TIEn		0x80000000
#define BERRIE	 	0x40000000
#define BCLROE	 	0x20000000
#define BCLRIE	 	0x10000000
#define SWAP	 	0x08000000
#define FAST		0x04000000
#define ArbIDL 		0x03000000
#define TValue 		0x00f80000
#define TPres	 	0x00060000
#define Ten	 	0x00010000
#define RISDIS		0x00008000
#define AINC2		0x00004000
#define AINC1		0x00002000
#define IPins		0x00001000
#define INTRL		0x00000c00
#define _16BIT		0x00000200
#define ABUF		0x00000100

/* define interrupts levels */
#define NO_INTR		0x00
#define INTR_LVL_1	0x01
#define INTR_LVL_6	0x02
#define INTR_LVL_7	0x03


/* PCWER - PCMCIA Configuration Register Write Event Register */
#define IOEIR		0x10
#define SCR		0x08
#define PRR		0x04
#define CCSR		0x02
#define COR		0x01

/* PCAWER - PCMCIA Access Wake-up Event Register */
#define MRW		0x10
#define CMS2		0x08
#define CMS1		0x04
#define ATS		0x02
#define IOS		0x01

/* PCMCIA event register */

#define SW		0x04
#define PTIE		0x02
#define BERR		0x01


/* PCMCIA CMBAR/CISBAR reg */

#define BAR_ADDR 	0xfff0 	/* bits 12 - 23 */
#define FC		0x000e
#define VALID		0x0001


/* Card configuration Registers */

/* COR - Reg */
#define SRESET		0x80
#define LevIREQ		0x40
#define ConfIndx	0x3f

/* Card Configuration and status register */
#define CHANGED		0x80
#define SigChg		0x40
#define IOis8		0x20
#define Ring_En		0x10
#define Audio		0x08
#define PwrDwn		0x04
#define Intr		0x02

/* PPR - Pin Replacment Register */
#define CBVD1		0x80
#define CBVD2		0x40
#define Crdy_Busy	0x20
#define CWprot		0x10
#define RBVD1		0x08
#define RBVD2		0x04
#define Rrdy_Busy	0x02
#define REProt		0x01

/* SCR - Socket and Copy Register */
#define CPYNUM		0x70
#define SOCKNUM		0x0f

/* IOEIR- IO Event Indication Register */
#define PIEvt		0x20
#define RIEvt		0x10
#define ResEnab3	0x08
#define ResEnab2	0x04
#define PIEnab		0x02
#define RIEnab		0x01
#endif
