/****************************************************************/
/*								*/
/*								*/
/* (C) Copyright Motorola Inc, 1994.				*/
/*  All right reserved 						*/
/*                                                              */
/* Author: Uri Trichter                                         */
/*								*/
/* Description:							*/
/*     Simple Driver Set - Application Test routine 		*/
/*								*/
/*     This application is initializing scc0 to be UART 	*/
/*     controller with 7 bits data, even parity, one stop bit.	*/
/*     it sends "Hello World" on the connection and waits to 	*/
/*     receive characters. Every character that it gets it echos*/
/*     back on the connection. The partenr can be terminal 	*/
/*     emulation running on a PC sending files			*/
/*								*/
/*								*/
/****************************************************************/

#include "simple.h"
#include "setup.h"
#include "uart.h"

#include <stdio.h>
#include <stdlib.h>

#define IMP_START_ADDR	0x700000L

#define TRUE	1
#define FALSE	0

#define CTL_Q	0x11
#define CTL_S	0x13

static stop_transmit = FALSE;

void got_a_buffer(int scc_number, void *buff, int length)
{
	/* echo the same buffer back to the transmitter */
	/* Transmit done interrupt will take care of freeing this buffer */
	/* In case you got CTL_S free the buffer and do not transmit it */
	
	if( !stop_transmit){
		while( uart_tx(scc_number, buff, length) == BUSY )
			;
	}else
	 	free(buff);
}

void rx_errors(int scc_number, IMP_BD *rbd)
{
	/* just record statistics about receive errors that accure */
	static int brk = 0;
	static int framing_err = 0;
	static int parity_err = 0;
	static int carrier_detect = 0;

	if(rbd->status& R_BRK)
		brk++;
	if(rbd->status& R_FR)
		framing_err++;
	if(rbd->status& R_PR)
		parity_err++;
	if(rbd->status& R_CD)
		carrier_detect++;

}

void busy_errors(int scc_number)
{
	int length;
	IMP_BD	*bd;
	static char busy_cond[]= "You Have A Busy Condition!! Check Performance \r\n";
	static char mem_prob[] = "System Can not allocate memory!! Check Heap Size\r\n";
	char buf[60];

	/* first check if it is a problem becuase the receiver
	   Can not malloc memory to replenish Rx buffers.
	*/

	bd = scc_table[scc_number].rbd;
	if( bd ->length == 0 ){
		memcpy( buf,mem_prob, (length=strlen(mem_prob)));
		/* inform user using the RS232 connection */
		while( uart_tx(scc_number, buf, length) == BUSY )
				;
		/* try to allocate memory again */
		alloc_mem(scc_number,bd);
	}else {
		memcpy( buf,busy_cond, (length=strlen(busy_cond)));
		/* inform user using the RS232 connection */
		while( uart_tx(scc_number, buf, length) == BUSY )
				;
	}
		
}


void receive_ctl_s()
{
	stop_transmit = TRUE;	
}

void receive_ctl_q()
{
	stop_transmit = FALSE;	
}
/* write user interrupt service routine */
/* This is Microtech way to end a routine with RTE command */ 

void interrupt scc_isr_0()
{
	scc_interrupt( SCC_0 );
}

void interrupt scc_isr_1()
{
	scc_interrupt( SCC_1 );
}

void interrupt scc_isr_2()
{
	scc_interrupt( SCC_2 );
}


main()
{
	char buf[] = "Hello World\r\n";

	SETUP set;

	/* initiate IMP */
	imp_init(IMP_START_ADDR);
	
	/* init the scc */
	scc_init( SCC_0, 	
		16,
		malloc,
		free,
		got_a_buffer,
		0x00L,		/* no transmit error handling in UART */
		rx_errors,
		busy_errors); 


	/* init communication parameters */

	set.comm_sett.baud = 9600;
	set.comm_sett.data_bits = SEVEN_BIT_DATA;
	set.comm_sett.parity = EVEN_PAR;
	set.comm_sett.stop_bits = ONE_STOP_B;

	/* init cc table */
	set.cc_oper.num_of_cc_to_handle = 2;
	set.cc_oper.cc_table[0].reject = TRUE;
	set.cc_oper.cc_table[0].cc = CTL_Q;
	set.cc_oper.cc_table[0].cc_func_oper = receive_ctl_q;
	set.cc_oper.cc_table[1].reject = TRUE;
	set.cc_oper.cc_table[1].cc = CTL_S;
	set.cc_oper.cc_table[1].cc_func_oper = receive_ctl_s;

  	/* init cannel to be UART */ 
	uart_init(SCC_0, &set);

	while( uart_tx(SCC_0, buf, strlen(buf)) == BUSY )
			;

	for(;;)
		;
	
}
